#include "vtkPNMReader.h"
#include "vtkTexture.h"
#include "vtkRenderer.h"
#include "vtkQtRenderWindow.h"
#include "vtkRenderer.h"
#include "vtkSphereSource.h"
#include "vtkPolyDataMapper.h"
#include "vtkActor.h"
#include "vtkConeSource.h"
#include "vtkGlyph3D.h"
#include "vtkPolyData.h"
#include "vtkQtRenderWindowInteractor.h"

int main( int argc, char *argv[] ) {
    vtkRenderer *ren1 = vtkRenderer::New();
    vtkRenderer *ren2 = vtkRenderer::New();
    vtkQtRenderWindow *renWin = vtkQtRenderWindow::New();
    renWin->AddRenderer(ren1);
    renWin->AddRenderer(ren2);

    vtkRenderer *ren3 = vtkRenderer::New();
    vtkQtRenderWindow *renWin2 = vtkQtRenderWindow::New();
    renWin2->AddRenderer(ren3);

    vtkSphereSource *sphere = vtkSphereSource::New();
    sphere->SetThetaResolution(8); sphere->SetPhiResolution(8);
    vtkPolyDataMapper *sphereMapper = vtkPolyDataMapper::New();
    sphereMapper->SetInput(sphere->GetOutput());
    vtkActor *sphereActor1 = vtkActor::New();
    sphereActor1->SetMapper(sphereMapper);
    vtkActor *sphereActor2 = vtkActor::New();
    sphereActor2->SetMapper(sphereMapper);
    vtkActor *sphereActor3 = vtkActor::New();
    sphereActor3->SetMapper(sphereMapper);

    vtkConeSource *cone = vtkConeSource::New();
    cone->SetResolution(6);

    vtkGlyph3D *glyph = vtkGlyph3D::New();
    glyph->SetInput(sphere->GetOutput());
    glyph->SetSource(cone->GetOutput());
    glyph->SetVectorModeToUseNormal();
    glyph->SetScaleModeToScaleByVector();
    glyph->SetScaleFactor(0.25);

    vtkPolyDataMapper *spikeMapper = vtkPolyDataMapper::New();
    spikeMapper->SetInput(glyph->GetOutput());

    vtkPNMReader *reader = vtkPNMReader::New();
    reader->SetFileName("masonry.ppm");
    vtkTexture *text = vtkTexture::New();
    text->SetInput(reader->GetOutput());
    
    vtkActor *spikeActor1 = vtkActor::New();
    spikeActor1->SetTexture(text);
    spikeActor1->SetMapper(spikeMapper);
    vtkActor *spikeActor2 = vtkActor::New();
    spikeActor2->SetMapper(spikeMapper);
    vtkActor *spikeActor3 = vtkActor::New();
    spikeActor3->SetMapper(spikeMapper);

    ren1->AddActor(sphereActor1);
    ren1->AddActor(spikeActor1);
    ren1->SetBackground(0.4,0.1,0.2);
    ren1->SetViewport(0,0,0.5,1.0);

    ren2->AddActor(sphereActor2);
    ren2->AddActor(spikeActor2);
    ren2->SetBackground(0.1,0.2,0.4);
    ren2->SetViewport(0.5,0,1.0,1.0);
    renWin->SetSize(300,150);
    renWin->SetPosition(0, 400);

    ren3->AddActor(sphereActor3);
    ren3->AddActor(spikeActor3);
    ren3->SetBackground(0.1,0.4,0.2);
    renWin2->SetSize(300,300);
    renWin2->SetPosition(0, 50);

    // allow keyboard manipulation of object
    vtkQtRenderWindowInteractor *iren = vtkQtRenderWindowInteractor::New();
    iren->SetRenderWindow(renWin);
    renWin->Render();

    vtkQtRenderWindowInteractor *iren2 = vtkQtRenderWindowInteractor::New();
    iren2->SetRenderWindow(renWin2);
    renWin2->Render();

    iren->Start();

    // Clean up
    ren1->Delete();
    ren2->Delete();
    renWin->Delete();
    ren3->Delete();
    renWin2->Delete();
    sphere->Delete();
    sphereMapper->Delete();
    sphereActor1->Delete();
    sphereActor2->Delete();
    sphereActor3->Delete();
    cone->Delete();
    glyph->Delete();
    spikeMapper->Delete();
    spikeActor1->Delete();
    spikeActor2->Delete();
    spikeActor3->Delete();
    iren->Delete();
    iren2->Delete();

    return 0;
}


