#include <qapplication.h>
#include "qvExample.h"

#include "vtkRenderer.h"
#include "vtkStructuredPoints.h"
#include "vtkStructuredPointsReader.h"
#include "vtkPiecewiseFunction.h"
#include "vtkVolumeProperty.h"
#include "vtkVolumeRayCastCompositeFunction.h"
#include "vtkVolumeRayCastMapper.h"
#include "vtkVolume.h"
#include "vtkConeSource.h"
#include "vtkPolyDataMapper.h"
#include "vtkActor.h"
#include "vtkCamera.h"

vtkRenderer* makeRendererLeft() {
    vtkRenderer *renderer = vtkRenderer::New();

    vtkStructuredPointsReader *reader = vtkStructuredPointsReader::New();
    reader->SetFileName("e:/vtk42/VTKData/Data/ironProt.vtk");
    reader->Update();
    vtkPiecewiseFunction *oTFun = vtkPiecewiseFunction::New();
    oTFun->AddSegment(80, 0.0, 255, 1.0);

    vtkPiecewiseFunction *cTFun = vtkPiecewiseFunction::New();
    cTFun->AddSegment(0, 1.0, 255, 1.0);

    vtkVolumeProperty *volumeProperty = vtkVolumeProperty::New();
    volumeProperty->SetColor(cTFun);
    volumeProperty->SetScalarOpacity(oTFun);
    volumeProperty->SetInterpolationTypeToLinear();
    volumeProperty->ShadeOn();

    vtkVolumeRayCastCompositeFunction *compositeFunction = 
        vtkVolumeRayCastCompositeFunction::New();

    vtkVolumeRayCastMapper *volumeMapper = vtkVolumeRayCastMapper::New();
    volumeMapper->SetInput(reader->GetOutput());
    volumeMapper->SetVolumeRayCastFunction(compositeFunction);

    vtkVolume *volume = vtkVolume::New();
    volume->SetMapper(volumeMapper);
    volume->SetProperty(volumeProperty);

    renderer->AddVolume(volume);
    renderer->GetActiveCamera()->Azimuth(20.0);
    renderer->GetActiveCamera()->Dolly(1.60);
    renderer->ResetCameraClippingRange();
 
    return renderer;
}

vtkRenderer* makeRendererRight() {
    vtkRenderer *renderer = vtkRenderer::New();

    vtkConeSource *cone = vtkConeSource::New();
    cone->SetResolution(8);

    vtkPolyDataMapper *coneMapper = vtkPolyDataMapper::New();
    coneMapper->SetInput(cone->GetOutput());

    vtkActor *coneActor = vtkActor::New();
    coneActor->SetMapper(coneMapper);
    
    renderer->AddActor(coneActor);
    return renderer;
}

int main (int argc, char *argv[]) {
    QApplication::setColorSpec(QApplication::CustomColor);
    QApplication a(argc, argv);

    QVExample *qve = new QVExample();
    qve->setLeftRenderer(makeRendererLeft());
    qve->setRightRenderer(makeRendererRight());
    qve->show();
    
    a.setMainWidget(qve);
    int ret = a.exec();
    
}
