/*=========================================================================
  vtkQtRenderWindow.cpp - copyright 2001 Matthias Koenig 
  koenig@isg.cs.uni-magdeburg.de
  http://wwwisg.cs.uni-magdeburg.de/~makoenig
  =========================================================================*/
/*=========================================================================
  This module is an extension of the "Visualization Toolkit 
  ( copyright (c) 1993-2000 Ken Martin, Will Schroeder, Bill Lorensen )".
  and combines it with "Qt (copyright (C) 1992-2000 Troll Tech AS)".
  =========================================================================*/
/*=========================================================================

  Author:    Matthias Koenig, last changed by $Author: docbill $
  Module:    $RCSfile: vtkQtRenderWindow.cpp,v $
  Date:      $Date: 2005/11/09 21:21:42 $
  Version:   $Revision: 1.5 $

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
  * Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
   
  * Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
   
  * Modified source versions must be plainly marked as such, and must not be
  misrepresented as being the original software.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  
  =========================================================================*/

#include "vtkQtRenderWindow.h"
#include "vtkQtRenderWindowInteractor.h"
 
#define MAX_LIGHTS 8

#ifdef USE_REPARENT_FIX
// this is the widget which will delete the vtkQtRenderWindow
class vtkQtAutoDelete : private QObject
{
public:
  vtkQtAutoDelete(QWidget *parent,vtkQtRenderWindow *xvtkChild);
  // delete both this object and the child, if there still is one.
  virtual ~vtkQtAutoDelete();
  // delete this object, if there is still a child.
  void destroy();
  // destroy this object and replace it with one that has a new parent
  vtkQtAutoDelete* reparent(QWidget *p);
  vtkQtRenderWindow *vtkChild;
  static QWidget *orphan();
};
#endif // USE_REPARENT_FIX

QApplication* vtkQtRenderWindow::qtapp = NULL;

void vtkQtRenderWindow::initApp() {
  if (qApp)
    return;
  int ac = 0; char **av = NULL;
  qtapp = new QApplication(ac, av);
}

vtkQtRenderWindow* vtkQtRenderWindow::New(QWidget *parent, const char* name , const QGLWidget *shareWidget, WFlags f) {
  initApp();
  vtkQtRenderWindow* tmp = new vtkQtRenderWindow(parent, name, shareWidget, f);
  tmp->show();
  return tmp;
}

vtkQtRenderWindow::vtkQtRenderWindow(const QGLFormat &format, QWidget *parent, const char *name, const QGLWidget *shareWidget, WFlags f)
  :QGLWidget (format, parent, name, shareWidget, f), ptr(0) {
#ifdef USE_REPARENT_FIX
  reparent(parent,f);
#endif // USE_REPARENT_FIX
  initvtkQtRenderWindow();
}

vtkQtRenderWindow::vtkQtRenderWindow(QWidget *parent, const char *name, const QGLWidget *shareWidget, WFlags f)
  :QGLWidget (parent, name, shareWidget, f), ptr(0) {
#ifdef USE_REPARENT_FIX
  reparent(parent,f);
#endif // USE_REPARENT_FIX
  initvtkQtRenderWindow();
}

void vtkQtRenderWindow::initvtkQtRenderWindow() {
  vtkDebugMacro(<< " vtkQtRenderWindow constructor\n");
#ifdef _WIN32
  WindowId = QWidget::winId();
  DeviceContext = GetDC(WindowId);
  this->MultiSamples = 8;
  this->SetWindowName("Visualization Toolkit - Qt+OpenGL");
#else
  DisplayId = QPaintDevice::x11AppDisplay();
  ColorMap = QPaintDevice::x11AppColormap();
  WindowId = QPaintDevice::handle();
  #ifndef VTK_USE_MESA
    this->MultiSamples = GetGlobalMaximumNumberOfMultiSamples();
  #endif
  if ( this->WindowName ) 
    delete[] this->WindowName;
  this->WindowName = new char[strlen("Visualization Toolkit - Qt+OpenGL")+1];
  strcpy( this->WindowName, "Visualization Toolkit - Qt+OpenGL" );
  setCaption(this->WindowName);
#endif
  SetSwapBuffers(false);
  this->Interactor = NULL;
  qtRenWinInt = NULL;
  setFocusPolicy(QWidget:: WheelFocus);
}

vtkQtRenderWindow::~vtkQtRenderWindow() {
#ifdef USE_REPARENT_FIX
  vtkQtAutoDelete *tmpptr=ptr;
  ptr=0;
  vtkQtRenderWindow *v=this;
  if(tmpptr&&(tmpptr->vtkChild == v)) tmpptr->destroy();
#endif // USE_REPARENT_FIX

  short cur_light;
  vtkOpenGLRenderer *ren;
  
  makeCurrent();
  for (cur_light = GL_LIGHT0; cur_light < GL_LIGHT0+MAX_LIGHTS; cur_light++) {
    glDisable((GLenum)cur_light);
  }

  // tell each of the renderers that this render window/graphics context
  // is being removed (the RendererCollection is removed by vtkRenderWindow's
  // destructor)
  this->Renderers->InitTraversal();
  for ( ren = (vtkOpenGLRenderer *) this->Renderers->GetNextItemAsObject();
        ren != NULL;
        ren = (vtkOpenGLRenderer *) this->Renderers->GetNextItemAsObject() ) {
    ren->SetRenderWindow(NULL);
  }
  glFinish();

#ifndef USE_REPARENT_FIX // Warnings are better than illegal access violations
  // to circumvent VTK warnings; if this destructor is called,
  // we believe that it is correct and delete without taking
  // care of the reference count
  if (this->ReferenceCount > 0) {
      this->SetReferenceCount(0);
  }
#endif // USE_REPARENT_FIX
}

// Initialize the window for rendering.
void vtkQtRenderWindow::initializeGL (void) {
  vtkDebugMacro(<< " InitializeGL\n");
    
  vtkDebugMacro(<< " glMatrixMode ModelView\n");
  glMatrixMode( GL_MODELVIEW );
    
  vtkDebugMacro(<< " zbuffer enabled\n");
  glDepthFunc( GL_LEQUAL );
  glEnable( GL_DEPTH_TEST );
    
  vtkDebugMacro(" texture stuff\n");
  glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
    
  // initialize blending for transparency
  vtkDebugMacro(<< " blend func stuff\n");
  glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
  glEnable(GL_BLEND);
    
  glEnable( GL_NORMALIZE );
  glAlphaFunc(GL_GREATER,0);
    
  this->Mapped = 1;
}

void vtkQtRenderWindow::resizeGL(int x,int y) {
  vtkDebugMacro(<< " SetSize\n");
  if ((this->Size[0] != x)||(this->Size[1] != y)) {
    this->Modified();
    this->Size[0] = x;
    this->Size[1] = y;
  }
}

void vtkQtRenderWindow::PrintSelf(ostream& os, vtkIndent indent) {
  this->vtkRenderWindow::PrintSelf(os,indent);
  os << indent << "MultiSamples: " << this->MultiSamples << "\n";
}

void vtkQtRenderWindow::paintGL() {
  vtkDebugMacro(<< " paintGL()\n");
  vtkRenderWindow::Render();
}

vtkRenderWindowInteractor* vtkQtRenderWindow::MakeRenderWindowInteractor() {
  qtRenWinInt = vtkQtRenderWindowInteractor::New();
  this->Interactor = qtRenWinInt;
  qtRenWinInt->SetRenderWindow(this);
  return qtRenWinInt;
}

void vtkQtRenderWindow::SetInteractor(vtkQtRenderWindowInteractor* arenwin) {
  qtRenWinInt = arenwin;
  vtkRenderWindow::SetInteractor(arenwin);
}

void vtkQtRenderWindow::mousePressEvent(QMouseEvent *me) {
  if (qtRenWinInt)
    qtRenWinInt->mousePressEvent(me);
}

void vtkQtRenderWindow::mouseReleaseEvent(QMouseEvent *me) {
  if (qtRenWinInt)
    qtRenWinInt->mouseReleaseEvent(me);
}

void vtkQtRenderWindow::mouseMoveEvent(QMouseEvent *me) {
  if (qtRenWinInt)
    qtRenWinInt->mouseMoveEvent(me);
}

void vtkQtRenderWindow::keyPressEvent(QKeyEvent *ke) {
  if (qtRenWinInt)
    qtRenWinInt->keyPressEvent(ke);
}

#ifdef USE_REPARENT_FIX
vtkQtAutoDelete::vtkQtAutoDelete(QWidget *parent,vtkQtRenderWindow *v)
: QObject(parent), vtkChild(v) {}

vtkQtAutoDelete::~vtkQtAutoDelete()
{
  vtkQtRenderWindow *v=vtkChild;
  vtkChild=0;
  if(v)
  {
    v->reparent(orphan());
    v->Delete();
  }
}

void
vtkQtAutoDelete::destroy()
{
  vtkQtRenderWindow *c=vtkChild;
  vtkChild=0;
  if(c) delete this;
}

vtkQtAutoDelete*
vtkQtAutoDelete::reparent(QWidget *p)
{
  vtkQtRenderWindow *c=vtkChild;
  if((!c)||(p == orphan()))
  {
    return 0;
  }
  if(p == parent())
  {
    return this;
  }
  destroy();
  return new vtkQtAutoDelete(p,c);
}

void
vtkQtRenderWindow::reparent(QWidget *parent,QWidget::WFlags f)
{
  QPoint p=pos();
  if(ptr)
  {
    reparent(parent,f,p);
  }
  else
  {
    QWidget *w=new QWidget();
    QGLWidget::reparent(w,f,p);
    ptr=new vtkQtAutoDelete(parent,this);
    QGLWidget::reparent(parent,f,p);
    delete w;
  }
}

void
vtkQtRenderWindow::reparent(
  QWidget *parent, QWidget::WFlags f,const QPoint &p,bool showIt)
{
  if(ptr) ptr=ptr->reparent(parent);
  QGLWidget::reparent(parent,f,p,showIt);
}

void
vtkQtRenderWindow::reparent(QWidget *parent,const QPoint &p,bool showIt) 
{
  reparent(parent,0,p,showIt);
}

QWidget *
vtkQtAutoDelete::orphan()
{
  static QWidget *retval=new QWidget();
  return retval;
}
#endif // USE_REPARENT_FIX


